@@ -13,13 +13,13 @@ RequestUpload = collections.namedtuple(
     ("path", "save_name", "artifact_id", "md5", "copied", "save_fn", "digest"),
 )
 RequestCommitArtifact = collections.namedtuple(
-    "RequestCommitArtifact", ("artifact_id", "before_commit", "on_commit")
+    "RequestCommitArtifact", ("artifact_id", "finalize", "before_commit", "on_commit")
 )
 RequestFinish = collections.namedtuple("RequestFinish", ())
 
 
 class StepUpload(object):
-    def __init__(self, api, stats, event_queue, max_jobs):
+    def __init__(self, api, stats, event_queue, max_jobs, silent=False):
         self._api = api
         self._stats = stats
         self._event_queue = event_queue
@@ -35,6 +35,7 @@ class StepUpload(object):
         self._artifacts = {}
 
         self._finished = False
+        self.silent = silent
 
     def _thread_body(self):
         # Wait for event in the queue, and process one by one until a
@@ -84,6 +85,7 @@ class StepUpload(object):
             if event.artifact_id not in self._artifacts:
                 self._init_artifact(event.artifact_id)
             self._artifacts[event.artifact_id]["commit_requested"] = True
+            self._artifacts[event.artifact_id]["finalize"] = event.finalize
             if event.before_commit:
                 self._artifacts[event.artifact_id]["pre_commit_callbacks"].add(
                     event.before_commit
@@ -121,6 +123,7 @@ class StepUpload(object):
             self._event_queue,
             self._stats,
             self._api,
+            self.silent,
             event.save_name,
             event.path,
             event.artifact_id,
@@ -148,7 +151,8 @@ class StepUpload(object):
         ):
             for callback in artifact_status["pre_commit_callbacks"]:
                 callback()
-            self._api.commit_artifact(artifact_id)
+            if artifact_status["finalize"]:
+                self._api.commit_artifact(artifact_id)
             for callback in artifact_status["post_commit_callbacks"]:
                 callback()
 
