@@ -7,4 +7,4 @@ In the future use:
 
 from wandb.integration.lightgbm import wandb_callback
 
-__all__ = ['wandb_callback']
+__all__ = ["wandb_callback"]
