@@ -210,6 +210,11 @@ class BackendSender(object):
         rec = self._make_record(history=history)
         self._publish(rec)
 
+    def publish_preempting(self) -> None:
+        preempt_rec = pb.RunPreemptingRecord()
+        rec = self._make_record(preempting=preempt_rec)
+        self._publish(rec)
+
     def publish_history(
         self, data: dict, step: int = None, run: "Run" = None, publish_step: bool = True
     ) -> None:
@@ -485,6 +490,7 @@ class BackendSender(object):
         footer: pb.FooterRecord = None,
         request: pb.Request = None,
         telemetry: tpb.TelemetryRecord = None,
+        preempting: pb.RunPreemptingRecord = None,
     ) -> pb.Record:
         record = pb.Record()
         if run:
@@ -519,6 +525,8 @@ class BackendSender(object):
             record.telemetry.CopyFrom(telemetry)
         elif metric:
             record.metric.CopyFrom(metric)
+        elif preempting:
+            record.preempting.CopyFrom(preempting)
         else:
             raise Exception("Invalid record")
         return record
@@ -538,6 +546,8 @@ class BackendSender(object):
 
     def _communicate_async(self, rec: pb.Record, local: bool = None) -> _Future:
         assert self._router
+        if self._process and not self._process.is_alive():
+            raise Exception("The wandb backend process has shutdown")
         future = self._router.send_and_receive(rec, local=local)
         return future
 
@@ -609,6 +619,7 @@ class BackendSender(object):
         val: Any = None,
     ) -> None:
         cfg = self._make_config(data=data, key=key, val=val)
+
         self._publish_config(cfg)
 
     def _publish_config(self, cfg: pb.ConfigRecord) -> None:
@@ -766,12 +777,10 @@ class BackendSender(object):
         assert result.exit_result
         return result.exit_result
 
-    def communicate_poll_exit(
-        self, timeout: int = None
-    ) -> Optional[pb.PollExitResponse]:
+    def communicate_poll_exit(self) -> Optional[pb.PollExitResponse]:
         poll_request = pb.PollExitRequest()
         rec = self._make_request(poll_exit=poll_request)
-        result = self._communicate(rec, timeout=timeout)
+        result = self._communicate(rec)
         if result is None:
             return None
         poll_exit_response = result.response.poll_exit_response
