@@ -65,12 +65,15 @@ class DataStore(object):
         self._opened_for_scan = False
         self._fp = None
         self._index = 0
+        self._size_bytes = 0
 
         self._crc = [0] * (LEVELDBLOG_LAST + 1)
         for x in range(1, LEVELDBLOG_LAST + 1):
             self._crc[x] = zlib.crc32(strtobytes(chr(x))) & 0xFFFFFFFF
 
-        assert wandb._assert_is_internal_process
+        assert (
+            wandb._assert_is_internal_process
+        ), "DataStore can only be used in the internal process"
 
     def open_for_write(self, fname):
         self._fname = fname
@@ -95,24 +98,36 @@ class DataStore(object):
         logger.info("open for scan: %s", fname)
         self._fp = open(fname, "rb")
         self._index = 0
+        self._size_bytes = os.stat(fname).st_size
         self._opened_for_scan = True
         self._read_header()
 
+    def in_last_block(self):
+        """When reading, we want to know if we're in the last block to
+           handle in progress writes"""
+        return self._index > self._size_bytes - LEVELDBLOG_DATA_LEN
+
     def scan_record(self):
-        assert self._opened_for_scan
+        assert self._opened_for_scan, "file not open for scanning"
         # TODO(jhr): handle some assertions as file corruption issues
         # assume we have enough room to read header, checked by caller?
         header = self._fp.read(LEVELDBLOG_HEADER_LEN)
         if len(header) == 0:
             return None
-        assert len(header) == LEVELDBLOG_HEADER_LEN
+        assert (
+            len(header) == LEVELDBLOG_HEADER_LEN
+        ), "record header is {} bytes instead of the expected {}".format(
+            len(header), LEVELDBLOG_HEADER_LEN
+        )
         fields = struct.unpack("<IHB", header)
         checksum, dlength, dtype = fields
         # check len, better fit in the block
         self._index += LEVELDBLOG_HEADER_LEN
         data = self._fp.read(dlength)
         checksum_computed = zlib.crc32(data, self._crc[dtype]) & 0xFFFFFFFF
-        assert checksum == checksum_computed
+        assert (
+            checksum == checksum_computed
+        ), "record checksum is invalid, data may be corrupt"
         self._index += dlength
         return dtype, data
 
@@ -125,7 +140,7 @@ class DataStore(object):
             pad_check = strtobytes("\x00" * space_left)
             pad = self._fp.read(space_left)
             # verify they are zero
-            assert pad == pad_check
+            assert pad == pad_check, "invald padding"
             self._index += space_left
 
         record = self.scan_record()
@@ -135,7 +150,9 @@ class DataStore(object):
         if dtype == LEVELDBLOG_FULL:
             return data
 
-        assert dtype == LEVELDBLOG_FIRST
+        assert (
+            dtype == LEVELDBLOG_FIRST
+        ), "expected record to be type {} but found {}".format(LEVELDBLOG_FIRST, dtype)
         while True:
             offset = self._index % LEVELDBLOG_BLOCK_LEN
             record = self.scan_record()
@@ -145,7 +162,11 @@ class DataStore(object):
             if dtype == LEVELDBLOG_LAST:
                 data += new_data
                 break
-            assert dtype == LEVELDBLOG_MIDDLE
+            assert (
+                dtype == LEVELDBLOG_MIDDLE
+            ), "expected record to be type {} but found {}".format(
+                LEVELDBLOG_MIDDLE, dtype
+            )
             data += new_data
         return data
 
@@ -156,13 +177,21 @@ class DataStore(object):
             LEVELDBLOG_HEADER_MAGIC,
             LEVELDBLOG_HEADER_VERSION,
         )
-        assert len(data) == 7
+        assert (
+            len(data) == LEVELDBLOG_HEADER_LEN
+        ), "header size is {} bytes, expected {}".format(
+            len(data), LEVELDBLOG_HEADER_LEN
+        )
         self._fp.write(data)
         self._index += len(data)
 
     def _read_header(self):
-        header_length = 7
-        header = self._fp.read(header_length)
+        header = self._fp.read(LEVELDBLOG_HEADER_LEN)
+        assert (
+            len(header) == LEVELDBLOG_HEADER_LEN
+        ), "header is {} bytes instead of the expected {}".format(
+            len(header), LEVELDBLOG_HEADER_LEN
+        )
         ident, magic, version = struct.unpack("<4sHB", header)
         if ident != strtobytes(LEVELDBLOG_HEADER_IDENT):
             raise Exception("Invalid header")
@@ -170,7 +199,6 @@ class DataStore(object):
             raise Exception("Invalid header")
         if version != LEVELDBLOG_HEADER_VERSION:
             raise Exception("Invalid header")
-        assert len(header) == header_length
         self._index += len(header)
 
     def _write_record(self, s, dtype=None):
@@ -179,7 +207,7 @@ class DataStore(object):
         # (this is a precondition to calling this method)
         assert len(s) + LEVELDBLOG_HEADER_LEN <= (
             LEVELDBLOG_BLOCK_LEN - self._index % LEVELDBLOG_BLOCK_LEN
-        )
+        ), "not enough space to write new records"
 
         dlength = len(s)
         dtype = dtype or LEVELDBLOG_FULL
@@ -220,7 +248,7 @@ class DataStore(object):
             self._write_record(s[:data_room], LEVELDBLOG_FIRST)
             data_used += data_room
             data_left -= data_room
-            assert data_left
+            assert data_left, "data_left should be non-zero"
 
             # write middles (if any)
             while data_left > LEVELDBLOG_DATA_LEN:
@@ -231,8 +259,10 @@ class DataStore(object):
                 data_used += LEVELDBLOG_DATA_LEN
                 data_left -= LEVELDBLOG_DATA_LEN
 
-            # write last
+            # write last and flush the entire block to disk
             self._write_record(s[data_used:], LEVELDBLOG_LAST)
+            self._fp.flush()
+            os.fsync(self._fp.fileno())
 
         return file_offset, self._index - file_offset, flush_index, flush_offset
 
@@ -249,7 +279,7 @@ class DataStore(object):
         """
         raw_size = obj.ByteSize()
         s = obj.SerializeToString()
-        assert len(s) == raw_size
+        assert len(s) == raw_size, "invalid serialization"
         ret = self._write_data(s)
         return ret
 
