@@ -1,4 +1,6 @@
 #
+import sys
+
 from pkg_resources import parse_version
 import requests
 import six
@@ -74,7 +76,6 @@ def check_available(current_version):
         return
 
     latest_version, pip_prerelease, deleted, yanked, yanked_reason = package_info
-
     upgrade_message = (
         "%s version %s is available!  To upgrade, please run:\n"
         " $ pip install %s --upgrade%s"
@@ -86,6 +87,15 @@ def check_available(current_version):
         )
     )
 
+    if sys.version_info[0] == 2 and parse_version(latest_version) >= parse_version(
+        "0.11.0"
+    ):
+        upgrade_message = "%s version %s available, but requires python3, see %s" % (
+            wandb._wandb_module,
+            latest_version,
+            "http://wandb.me/library-eol-python27",
+        )
+
     delete_message = None
     if deleted:
         delete_message = "%s version %s has been retired!  Please upgrade." % (
