@@ -11,12 +11,14 @@ import logging
 import numbers
 import os
 import platform
+import re
 import sys
 import threading
 import time
 import traceback
 
 import click
+import requests
 from six import iteritems, string_types
 from six.moves import _thread as thread
 from six.moves.collections_abc import Mapping
@@ -100,6 +102,7 @@ if wandb.TYPE_CHECKING:  # type: ignore
 logger = logging.getLogger("wandb")
 EXIT_TIMEOUT = 60
 RUN_NAME_COLOR = "#cdcd00"
+RE_LABEL = re.compile(r"[a-zA-Z0-9_-]+$")
 
 
 class ExitHooks(object):
@@ -113,6 +116,12 @@ class ExitHooks(object):
     def hook(self) -> None:
         self._orig_exit = sys.exit
         sys.exit = self.exit
+        self._orig_excepthook = (
+            sys.excepthook
+            if sys.excepthook
+            != sys.__excepthook__  # respect hooks by other libraries like pdb
+            else None
+        )
         sys.excepthook = self.exc_handler
 
     def exit(self, code: object = 0) -> "NoReturn":
@@ -139,6 +148,8 @@ class ExitHooks(object):
             self.exit_code = 255
 
         traceback.print_exception(exc_type, exc, tb)
+        if self._orig_excepthook:
+            self._orig_excepthook(exc_type, exc, tb)
 
 
 class RunStatusChecker(object):
@@ -760,6 +771,89 @@ class Run(object):
         """
         return self._entity or ""
 
+    def _label_internal(
+        self, code: str = None, repo: str = None, code_version: str = None
+    ) -> None:
+        with telemetry.context(run=self) as tel:
+            if code and RE_LABEL.match(code):
+                tel.label.code_string = code
+            if repo and RE_LABEL.match(repo):
+                tel.label.repo_string = repo
+            if code_version and RE_LABEL.match(code_version):
+                tel.label.code_version = code_version
+
+    def _label(
+        self,
+        code: str = None,
+        repo: str = None,
+        code_version: str = None,
+        **kwargs: str
+    ) -> None:
+        if self._settings.label_disable:
+            return
+        for k, v in (("code", code), ("repo", repo), ("code_version", code_version)):
+            if v and not RE_LABEL.match(v):
+                wandb.termwarn(
+                    "Label added for '{}' with invalid identifier '{}' (ignored).".format(
+                        k, v
+                    ),
+                    repeat=False,
+                )
+        for v in kwargs:
+            wandb.termwarn(
+                "Label added for unsupported key '{}' (ignored).".format(v),
+                repeat=False,
+            )
+
+        self._label_internal(code=code, repo=repo, code_version=code_version)
+
+        # update telemetry in the backend immediately for _label() callers
+        if self._backend:
+            self._backend.interface.publish_telemetry(self._telemetry_obj)
+
+    def _label_probe_lines(self, lines: List[str]) -> None:
+        if not lines:
+            return
+        parsed = telemetry._parse_label_lines(lines)
+        if not parsed:
+            return
+        label_dict = {}
+        code = parsed.get("code") or parsed.get("c")
+        if code:
+            label_dict["code"] = code
+        repo = parsed.get("repo") or parsed.get("r")
+        if repo:
+            label_dict["repo"] = repo
+        code_ver = parsed.get("version") or parsed.get("v")
+        if code_ver:
+            label_dict["code_version"] = code_ver
+        self._label_internal(**label_dict)
+
+    def _label_probe_main(self) -> None:
+        m = sys.modules.get("__main__")
+        if not m:
+            return
+        doc = getattr(m, "__doc__", None)
+        if not doc:
+            return
+
+        doclines = doc.splitlines()
+        self._label_probe_lines(doclines)
+
+    # TODO: annotate jupyter Notebook class
+    def _label_probe_notebook(self, notebook: Any) -> None:
+        logger.info("probe notebook")
+        lines = None
+        try:
+            data = notebook.probe_ipynb()
+            cell0 = data.get("cells", [])[0]
+            lines = cell0.get("source")
+        except Exception as e:
+            logger.info("Unable to probe notebook: {}".format(e))
+            return
+        if lines:
+            self._label_probe_lines(lines)
+
     def _repr_mimebundle_(
         self, include: Any = None, exclude: Any = None
     ) -> Dict[str, str]:
@@ -781,6 +875,9 @@ class Run(object):
             return
         self._backend.interface.publish_config(key=key, val=val, data=data)
 
+    def _set_config_wandb(self, key: str, val: Any) -> None:
+        self._config_callback(key=("_wandb", key), val=val)
+
     def _summary_update_callback(self, summary_record: SummaryRecord) -> None:
         if self._backend:
             self._backend.interface.publish_summary(summary_record)
@@ -1092,7 +1189,7 @@ class Run(object):
         base_path: Optional[str] = None,
         policy: str = "live",
     ) -> Union[bool, List[str]]:
-        """ Ensure all files matching *glob_str* are synced to wandb with the policy specified.
+        """ Ensure all files matching `glob_str` are synced to wandb with the policy specified.
 
         Arguments:
             glob_str: (string) a relative or absolute path to a unix glob or regular
@@ -1363,7 +1460,7 @@ class Run(object):
                 "{} {}".format(run_state_str, click.style(run_name, fg="yellow"))
             )
             emojis = dict(star="", broom="", rocket="")
-            if platform.system() != "Windows":
+            if platform.system() != "Windows" and sys.stdout.encoding == "UTF-8":
                 emojis = dict(star="⭐️", broom="🧹", rocket="🚀")
 
             wandb.termlog(
@@ -1404,12 +1501,6 @@ class Run(object):
         err_redir: redirect.RedirectBase
         if console == self._settings.Console.REDIRECT:
             logger.info("Redirecting console.")
-            # out_cap = redirect.Capture(
-            #     name="stdout", cb=self._redirect_cb, output_writer=self._output_writer
-            # )
-            # err_cap = redirect.Capture(
-            #     name="stderr", cb=self._redirect_cb, output_writer=self._output_writer
-            # )
             out_redir = redirect.Redirect(
                 src="stdout",
                 cbs=[
@@ -1471,27 +1562,6 @@ class Run(object):
             logger.error("Failed to redirect.", exc_info=e)
         return
 
-        # TODO(jhr): everything below here is not executed as we only support redir mode
-        #
-        # from wandb.lib import console as lib_console
-        # from wandb.old import io_wrap
-        #
-        # redirect stdout
-        # if platform.system() == "Windows":
-        #     lib_console.win32_redirect(stdout_slave_fd, stderr_slave_fd)
-        # else:
-        #     self._save_stdout = sys.stdout
-        #     self._save_stderr = sys.stderr
-        #     stdout_slave = os.fdopen(stdout_slave_fd, "wb")
-        #     stderr_slave = os.fdopen(stderr_slave_fd, "wb")
-        #     stdout_redirector = io_wrap.FileRedirector(sys.stdout, stdout_slave)
-        #     stderr_redirector = io_wrap.FileRedirector(sys.stderr, stderr_slave)
-        #     stdout_redirector.redirect()
-        #     stderr_redirector.redirect()
-        #     self.stdout_redirector = stdout_redirector
-        #     self.stderr_redirector = stderr_redirector
-        # logger.info("redirect done")
-
     def _restore(self) -> None:
         logger.info("restore")
         # TODO(jhr): drain and shutdown all threads
@@ -2248,13 +2318,19 @@ class Run(object):
     # TODO(jhr): annotate this
     def _assert_can_log_artifact(self, artifact) -> None:  # type: ignore
         if not self._settings._offline:
-            public_api = self._public_api()
-            expected_type = public.Artifact.expected_type(
-                public_api.client,
-                artifact.name,
-                public_api.settings["entity"],
-                public_api.settings["project"],
-            )
+            try:
+                public_api = self._public_api()
+                expected_type = public.Artifact.expected_type(
+                    public_api.client,
+                    artifact.name,
+                    public_api.settings["entity"],
+                    public_api.settings["project"],
+                )
+            except requests.exceptions.RequestException:
+                # Just return early if there is a network error. This is
+                # ok, as this function is intended to help catch an invalid
+                # type early, but not a hard requirement for valid operation.
+                return
             if expected_type is not None and artifact.type != expected_type:
                 raise ValueError(
                     "Expected artifact type {}, got {}".format(
@@ -2348,6 +2424,12 @@ class Run(object):
         self.finish(exit_code)
         return exc_type is None
 
+    def mark_preempting(self) -> None:
+        """Mark this run as preempting and tell the internal process
+        to immediately report this to the server."""
+        if self._backend:
+            self._backend.interface.publish_preempting()
+
 
 # We define this outside of the run context to support restoring before init
 def restore(
@@ -2460,7 +2542,11 @@ class _LazyArtifact(ArtifactInterface):
             if resp.error_message:
                 raise ValueError(resp.error_message)
             self._instance = public.Artifact.from_id(resp.artifact_id, self._api.client)
-        assert isinstance(self._instance, ArtifactInterface)
+        assert isinstance(
+            self._instance, ArtifactInterface
+        ), "Insufficient permissions to fetch Artifact with id {} from {}".format(
+            resp.artifact_id, self._api.client.app_url()
+        )
         return self._instance
 
     @property
